# flake8: noqa
# Form implementation generated from reading ui file './ui/main.ui'
#
# Created by: PyQt6 UI code generator 6.8.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(967, 600)
        self.centralwidget = QtWidgets.QWidget(parent=MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.lineEdit_session = QtWidgets.QLineEdit(parent=self.centralwidget)
        self.lineEdit_session.setReadOnly(True)
        self.lineEdit_session.setObjectName("lineEdit_session")
        self.gridLayout.addWidget(self.lineEdit_session, 1, 1, 1, 1)
        self.lineEdit_source = QtWidgets.QLineEdit(parent=self.centralwidget)
        self.lineEdit_source.setReadOnly(True)
        self.lineEdit_source.setObjectName("lineEdit_source")
        self.gridLayout.addWidget(self.lineEdit_source, 0, 1, 1, 1)
        self.label = QtWidgets.QLabel(parent=self.centralwidget)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(parent=self.centralwidget)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_26 = QtWidgets.QLabel(parent=self.centralwidget)
        self.label_26.setObjectName("label_26")
        self.horizontalLayout.addWidget(self.label_26)
        self.label_frame = QtWidgets.QLabel(parent=self.centralwidget)
        self.label_frame.setObjectName("label_frame")
        self.horizontalLayout.addWidget(self.label_frame)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.widget_labels = QtWidgets.QWidget(parent=self.centralwidget)
        self.widget_labels.setObjectName("widget_labels")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.widget_labels)
        self.horizontalLayout_2.setContentsMargins(-1, 0, -1, 0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_28 = QtWidgets.QLabel(parent=self.widget_labels)
        self.label_28.setObjectName("label_28")
        self.horizontalLayout_2.addWidget(self.label_28)
        self.label_label_current = QtWidgets.QLabel(parent=self.widget_labels)
        self.label_label_current.setObjectName("label_label_current")
        self.horizontalLayout_2.addWidget(self.label_label_current)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.label_labels_available = QtWidgets.QLabel(parent=self.widget_labels)
        self.label_labels_available.setObjectName("label_labels_available")
        self.horizontalLayout_2.addWidget(self.label_labels_available)
        self.horizontalLayout.addWidget(self.widget_labels)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout.addItem(spacerItem2)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.graphics_widget = GraphicsLayoutWidget(parent=self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.graphics_widget.sizePolicy().hasHeightForWidth())
        self.graphics_widget.setSizePolicy(sizePolicy)
        self.graphics_widget.setObjectName("graphics_widget")
        self.verticalLayout.addWidget(self.graphics_widget)
        self.widget = QtWidgets.QWidget(parent=self.centralwidget)
        self.widget.setObjectName("widget")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.widget)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.label_13 = QtWidgets.QLabel(parent=self.widget)
        self.label_13.setObjectName("label_13")
        self.gridLayout_2.addWidget(self.label_13, 5, 1, 1, 1)
        self.label_20 = QtWidgets.QLabel(parent=self.widget)
        self.label_20.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_20.setObjectName("label_20")
        self.gridLayout_2.addWidget(self.label_20, 9, 0, 1, 1)
        self.label_12 = QtWidgets.QLabel(parent=self.widget)
        self.label_12.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_12.setObjectName("label_12")
        self.gridLayout_2.addWidget(self.label_12, 5, 0, 1, 1)
        self.label_19 = QtWidgets.QLabel(parent=self.widget)
        self.label_19.setObjectName("label_19")
        self.gridLayout_2.addWidget(self.label_19, 8, 1, 1, 1)
        self.label_15 = QtWidgets.QLabel(parent=self.widget)
        self.label_15.setObjectName("label_15")
        self.gridLayout_2.addWidget(self.label_15, 6, 1, 1, 1)
        self.label_22 = QtWidgets.QLabel(parent=self.widget)
        self.label_22.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_22.setObjectName("label_22")
        self.gridLayout_2.addWidget(self.label_22, 10, 0, 1, 1)
        self.label_8 = QtWidgets.QLabel(parent=self.widget)
        self.label_8.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_8.setObjectName("label_8")
        self.gridLayout_2.addWidget(self.label_8, 3, 0, 1, 1)
        self.label_11 = QtWidgets.QLabel(parent=self.widget)
        self.label_11.setObjectName("label_11")
        self.gridLayout_2.addWidget(self.label_11, 4, 1, 1, 1)
        self.label_23 = QtWidgets.QLabel(parent=self.widget)
        self.label_23.setObjectName("label_23")
        self.gridLayout_2.addWidget(self.label_23, 10, 1, 1, 1)
        self.label_24 = QtWidgets.QLabel(parent=self.widget)
        self.label_24.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_24.setObjectName("label_24")
        self.gridLayout_2.addWidget(self.label_24, 11, 0, 1, 1)
        self.label_16 = QtWidgets.QLabel(parent=self.widget)
        self.label_16.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_16.setObjectName("label_16")
        self.gridLayout_2.addWidget(self.label_16, 7, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(parent=self.widget)
        self.label_4.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_4.setObjectName("label_4")
        self.gridLayout_2.addWidget(self.label_4, 2, 0, 1, 1)
        self.label_25 = QtWidgets.QLabel(parent=self.widget)
        self.label_25.setObjectName("label_25")
        self.gridLayout_2.addWidget(self.label_25, 11, 1, 1, 1)
        self.label_21 = QtWidgets.QLabel(parent=self.widget)
        self.label_21.setObjectName("label_21")
        self.gridLayout_2.addWidget(self.label_21, 9, 1, 1, 1)
        self.label_14 = QtWidgets.QLabel(parent=self.widget)
        self.label_14.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_14.setObjectName("label_14")
        self.gridLayout_2.addWidget(self.label_14, 6, 0, 1, 1)
        self.label_6 = QtWidgets.QLabel(parent=self.widget)
        self.label_6.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_6.setObjectName("label_6")
        self.gridLayout_2.addWidget(self.label_6, 1, 0, 1, 1)
        self.label_5 = QtWidgets.QLabel(parent=self.widget)
        self.label_5.setObjectName("label_5")
        self.gridLayout_2.addWidget(self.label_5, 2, 1, 1, 1)
        self.label_17 = QtWidgets.QLabel(parent=self.widget)
        self.label_17.setObjectName("label_17")
        self.gridLayout_2.addWidget(self.label_17, 7, 1, 1, 1)
        self.label_9 = QtWidgets.QLabel(parent=self.widget)
        self.label_9.setObjectName("label_9")
        self.gridLayout_2.addWidget(self.label_9, 3, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(parent=self.widget)
        self.label_3.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_3.setObjectName("label_3")
        self.gridLayout_2.addWidget(self.label_3, 0, 0, 1, 1)
        self.label_10 = QtWidgets.QLabel(parent=self.widget)
        self.label_10.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_10.setObjectName("label_10")
        self.gridLayout_2.addWidget(self.label_10, 4, 0, 1, 1)
        self.label_7 = QtWidgets.QLabel(parent=self.widget)
        self.label_7.setObjectName("label_7")
        self.gridLayout_2.addWidget(self.label_7, 1, 1, 1, 1)
        self.label_18 = QtWidgets.QLabel(parent=self.widget)
        self.label_18.setTextFormat(QtCore.Qt.TextFormat.MarkdownText)
        self.label_18.setObjectName("label_18")
        self.gridLayout_2.addWidget(self.label_18, 8, 0, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout_2.addItem(spacerItem3, 1, 2, 1, 1)
        self.verticalLayout.addWidget(self.widget)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(parent=MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 967, 23))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(parent=self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuHelp = QtWidgets.QMenu(parent=self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(parent=MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionOpen_rtdc_file = QtGui.QAction(parent=MainWindow)
        self.actionOpen_rtdc_file.setObjectName("actionOpen_rtdc_file")
        self.actionSegmentRrtdcFile = QtGui.QAction(parent=MainWindow)
        self.actionSegmentRrtdcFile.setObjectName("actionSegmentRrtdcFile")
        self.actionSegmentPngImages = QtGui.QAction(parent=MainWindow)
        self.actionSegmentPngImages.setObjectName("actionSegmentPngImages")
        self.actionQuit = QtGui.QAction(parent=MainWindow)
        self.actionQuit.setObjectName("actionQuit")
        self.actionExportImages = QtGui.QAction(parent=MainWindow)
        self.actionExportImages.setObjectName("actionExportImages")
        self.actionAbout = QtGui.QAction(parent=MainWindow)
        self.actionAbout.setObjectName("actionAbout")
        self.actionSoftware = QtGui.QAction(parent=MainWindow)
        self.actionSoftware.setObjectName("actionSoftware")
        self.actionDocumentation = QtGui.QAction(parent=MainWindow)
        self.actionDocumentation.setObjectName("actionDocumentation")
        self.menuFile.addAction(self.actionSegmentRrtdcFile)
        self.menuFile.addAction(self.actionSegmentPngImages)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionExportImages)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionQuit)
        self.menuHelp.addAction(self.actionAbout)
        self.menuHelp.addAction(self.actionSoftware)
        self.menuHelp.addAction(self.actionDocumentation)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.label.setText(_translate("MainWindow", "DC file"))
        self.label_2.setText(_translate("MainWindow", "Session"))
        self.label_26.setText(_translate("MainWindow", "Current frame:"))
        self.label_frame.setText(_translate("MainWindow", "--"))
        self.label_28.setText(_translate("MainWindow", "Current label:"))
        self.label_label_current.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:700\">--</span></p></body></html>"))
        self.label_labels_available.setText(_translate("MainWindow", "<html><head/><body><p>Frame Labels: --</p></body></html>"))
        self.label_13.setText(_translate("MainWindow", "Show previous/next image frame"))
        self.label_20.setText(_translate("MainWindow", "**B**"))
        self.label_12.setText(_translate("MainWindow", "Arrow keys (**← Left** or **→ Right**)"))
        self.label_19.setText(_translate("MainWindow", "Cycle through label visualization modes"))
        self.label_15.setText(_translate("MainWindow", "Zoom in/out"))
        self.label_22.setText(_translate("MainWindow", "**C**"))
        self.label_8.setText(_translate("MainWindow", "**Shift + Alt +Click**"))
        self.label_11.setText(_translate("MainWindow", "Change the current drawing label color"))
        self.label_23.setText(_translate("MainWindow", "Toggle auto contrast"))
        self.label_24.setText(_translate("MainWindow", "\"**.**\" (dot) and \"**-**\" (dash)"))
        self.label_16.setText(_translate("MainWindow", "**Space**"))
        self.label_4.setText(_translate("MainWindow", "**Shift + Click (and drag)**"))
        self.label_25.setText(_translate("MainWindow", "Modify label color saturation"))
        self.label_21.setText(_translate("MainWindow", "Toggle background correction (if available)"))
        self.label_14.setText(_translate("MainWindow", "**Mouse wheel** or **Right-Click (and drag)**"))
        self.label_6.setText(_translate("MainWindow", "**Click (and drag)**"))
        self.label_5.setText(_translate("MainWindow", "Remove pixels from a mask"))
        self.label_17.setText(_translate("MainWindow", "Temporarily hide the mask data"))
        self.label_9.setText(_translate("MainWindow", "Remove an entire mask"))
        self.label_3.setText(_translate("MainWindow", "**Shortcuts**"))
        self.label_10.setText(_translate("MainWindow", "Numbers (**1**, **2**, **3**, **4**, **5**, **6**, or **7**)"))
        self.label_7.setText(_translate("MainWindow", "Create or edit a mask. Holes are filled automatically"))
        self.label_18.setText(_translate("MainWindow", "**V**"))
        self.menuFile.setTitle(_translate("MainWindow", "File"))
        self.menuHelp.setTitle(_translate("MainWindow", "Help"))
        self.actionOpen_rtdc_file.setText(_translate("MainWindow", "Start/continue segmenting an .rtdc file..."))
        self.actionSegmentRrtdcFile.setText(_translate("MainWindow", "Segment .&rtdc file..."))
        self.actionSegmentPngImages.setText(_translate("MainWindow", "Segment .&png images..."))
        self.actionQuit.setText(_translate("MainWindow", "&Quit"))
        self.actionExportImages.setText(_translate("MainWindow", "&Export images..."))
        self.actionAbout.setText(_translate("MainWindow", "About"))
        self.actionSoftware.setText(_translate("MainWindow", "Software"))
        self.actionDocumentation.setText(_translate("MainWindow", "Documentation"))
from pyqtgraph import GraphicsLayoutWidget
